-module(xmerl_test).

-compile(export_all).
%%-export([Function/Arity, ...]).

-define(XMERL_APP,).

-include("xmerl.hrl").

test1() ->
    xmerl:export_simple(simple(), xmerl_html, [{title, "Doc Title"}]).


test2() ->
    xmerl:export_simple(simple(), xmerl_xml, [{title, "Doc Title"}]).


test3() ->
    FetchFun=fun(DTDSpec,S) -> {ok,not_fetched,S} end,
    {A,_}=xmerl_scan:string(html(),
			    [{fetch_fun,FetchFun}]),
    io:format("From xmerl_scan:string/2~n ~p~n",[A]),
    B=xmerl:export([A],xmerl_html),
    io:format("From xmerl:export/2 xmerl_html filter~n ~p~n",[B]),
    C=xmerl:export([A],xmerl_text),
    io:format("From xmerl:export/2 xmerl_text filter~n ~p~n",[C]).
    
test4() ->
    FetchFun=fun(DTDSpec,S) -> {ok,not_fetched,S} end,
    {A,_}=xmerl_scan:string(xml_namespace(),
			    [{fetch_fun,FetchFun},
			     {namespace_conformant,true}]),
    io:format("From xmerl_scan:string/2~n ~p~n",[A]).

test5() ->
    {ok, Cwd} = file:get_cwd(), % Assume we are in the examples dir...
    File=Cwd++"/xml/xmerl.xml",
    FetchFun=fun(DTDSpec,S) -> {ok,not_fetched,S} end,
%    {Resp0,Rest0}=xmerl_eventp:stream(File,[]),
%    io:format("Tree: ~p~n",[Resp0]),
    {Resp1,Rest1}=xmerl_eventp:file_sax(File,?MODULE,undefined,
					[{fetch_fun,FetchFun}]),
    io:format("Using file_sax: counted ~p  paragraphs~n",[Resp1]),
    {Resp2,Rest2}=xmerl_eventp:stream_sax(File,?MODULE,undefined,[]),
    io:format("Using stream_sax: counted ~p paragraphs~n",[Resp2]).

test6() ->
    FetchFun=fun(DTDSpec,S) -> {ok,{string,""},S} end,
    {Doc,_}=xmerl_scan:string(xml_namespace(),
			      [{fetch_fun,FetchFun},
			       {namespace_conformant,true}]),
    E=xmerl_xpath:string("child::title[position()=1]",Doc),
    io:format("From xmerl_scan:string/2~n E=~p~n",[E]).


simple() ->
    [{document, 
      [{title, ["Doc Title"]},
       {author, ["Ulf Wiger"]},
       {section,[{heading, ["heading1"]},
		 {'P', ["This is a paragraph of text."]},
		 {section,[{heading, ["heading2"]},
			   {'P', ["This is another paragraph."]},
			   {table,[{border, ["1"]}, 
				   {heading,[{col, ["head1"]},
					     {col, ["head2"]}]},
				   {row, [{col, ["col11"]},
					  {col, ["col12"]}]},
				   {row, [{col, ["col21"]},
					  {col, ["col22"]}]}
				  ]}
			  ]}
		]}
      ]}
    ].


html() ->
    "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"DTD/xhtml1-strict.dtd\"><html>"
	"<head><title>Doc Title</title><author>Ulf Wiger</author></head>"
	"<h1>heading1</h1>"
	"<p>This is a paragraph of text.</p>"
	"<h2>heading2</h2>"
	"<p>This is another paragraph.</p>"
	"<table>"
	"<thead><tr><td>head1</td><td>head2</td></tr></thead>"
	"<tr><td>col11</td><td>col122</td></tr>"
	"<tr><td>col21</td><td>col122</td></tr>"
	"</table>"
	"</html>".

xml_namespace() ->
    "<?xml version=\"1.0\"?>"
	"<!-- initially, the default namespace is \"books\" -->"
	"<book xmlns='urn:loc.gov:books' xmlns:isbn='urn:ISBN:0-395-36341-6'>"
	"<title>Cheaper by the Dozen</title>"
	"<isbn:number>1568491379</isbn:number>"
	"<notes>"
	"<!-- make HTML the default namespace for some comments -->"
	"<p xmlns='urn:w3-org-ns:HTML'>"
	"This is a <i>funny</i> book!"
	"</p>"
	"</notes>"
	"</book>".


%%% ============================================================================
%%% Generic callbacks

%'#text#'(Text) ->
%    [].

'#root#'(Data, Attrs, [], _E) ->
    io:format("root... Data=~p Attrs=~p E=~p~n",[Data,Attrs,_E]),
    [].

'#element#'(Tag, Data, Attrs, _Parents, _E) ->
    io:format("Tag=~p~n Data=~p~n Attrs=~p~n Parents=~p~n E=~p~n",
	      [Tag, Data, Attrs, _Parents, _E]),
    [].

'#element#'(Tag, Data, Attrs, CBstate) ->
%    io:format("Tag=~p~n Data=~p~n Attrs=~p~n CBstate=~p~n",
%	      [Tag, Data, Attrs, CBstate]),
    CBstate.

'#text#'(Text, CBstate) ->
    io:format("Text=~p~n CBstate=~p~n",
	      [Text, CBstate]),
    CBstate.


'#xml-inheritance#'() ->
    [xmerl_html].




%%% ============================================================================
%%% To run these tests you must first download the testsuite from www.w3c.org
%%% xmlconf.xml is the main test file that contains references to all the tests.
%%% Thus parse this, export result and execute tests in the call-back functions.
w3cvalidate() ->
    {A,Res}=xmerl_scan:file("xmlconf.xml",[]),
%    io:format("From xmerl:scan_file/2 ~n A=~p~n Res=~w~n",[A,Res]),
    C=xmerl:export([A],xmerl_test),
    io:format("From xmerl:export/2 xmerl_text filter~n ~p~n",[C]).
    

'TESTSUITE'(Data, Attrs, _Parents, _E) ->
    Profile=find_attribute('PROFILE',Attrs),
%    io:format("testsuite Profile=~p~n",[Profile]),
    [].

'TESTCASES'(Data, Attrs, _Parents, _E) ->
    Profile=find_attribute('PROFILE',Attrs),
    XMLbase=find_attribute('xml:base',Attrs),
    io:format("testsuite Profile=~p~n xml:base=~p~n",[Profile,XMLbase]),
    [].

'TEST'(Data, Attrs, _Parents, E) ->
%    io:format("test Attrs=~p~n Parents=~p~n E=~p~n",[Attrs, _Parents, E]),
    Id=find_attribute('ID',Attrs),
    io:format("Test: ~p ",[Id]),
    Entities=find_attribute('ENTITIES',Attrs),
    Output=find_attribute('OUTPUT',Attrs),
    Output3=find_attribute('OUTPUT3',Attrs),
    Sections=find_attribute('SECTIONS',Attrs),
    Type=find_attribute('TYPE',Attrs),
    URI=find_attribute('URI',Attrs),
    Namespace=find_attribute('NAMESPACE',Attrs),
    Test=filename:join(E#xmlElement.directory,URI),
    case Type of
	"valid" ->
%	    io:format("Data=~p~n Attrs=~p~n Parents=~p~n Path=~p~n",
%		      [Data, Attrs, _Parents, Test]),
	    test_valid(Test,Data);
	"invalid" ->
	    test_invalid(Test,Data);
	"not-wf" ->
	    test_notwf(Test,Data);
	"error" ->
	    test_error(Test,Data)
    end,
    [].


find_attribute(Tag,Attrs) ->
    case xmerl_lib:find_attribute(Tag,Attrs) of
	{value,Id} -> Id;
	false -> undefined
    end.



%%% All parsers must accept "valid" testcases.
test_valid(URI,Data) ->
    io:format("nonvalidating ",[]),
    case nonvalidating_parser(URI) of
	{Res,_Tail} when record(Res,xmlElement) ->
	    io:format("OK ",[]),
	    ok;
	Error ->
	    io:format("ERROR ~p~n URI=~p~n",[Error,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test})
    end,
    io:format("validating ",[]),
    case validating_parser(URI) of
	{Res2,_Tail2} when record(Res2,xmlElement) ->
	    io:format("OK~n",[]),
	    ok;
	Error2 ->
	    io:format("ERROR~n ~p~n URI=~p~n",[Error2,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test})
    end.


%%% Nonvalidating parsers must accept "invalid" testcases, but validating ones
%%% must reject them.
test_invalid(URI,Data) ->
    io:format("nonvalidating ",[]),
    case nonvalidating_parser(URI) of
	{Res,_Tail} when record(Res,xmlElement) ->
	    io:format("OK ",[]),
	    ok;
	Error ->
	    io:format("ERROR~n ~p~n URI=~p~n",[Error,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test})
    end,
    io:format("validating ",[]),
    case validating_parser_q(URI) of
	{Error2,_Tail2} when record(Error2,xmlElement) ->
	    io:format("ERROR~n ~p~n URI=~p~n",[Error2,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test});
	_Res2 ->
	    io:format("OK~n",[]),
	    ok
    end.

%%% No parser should accept a "not-wf" testcase unless it's a nonvalidating
%%% parser and the test contains external entities that the parser doesn't read
test_notwf(URI,Data) ->
    io:format("nonvalidating ",[]),
    case nonvalidating_parser_q(URI) of
	{Error,_Tail} when record(Error,xmlElement) ->
	    io:format("ERROR ~p~n URI=~p~n",[Error,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test});
	_Res ->
	    io:format("OK ",[]),
	    ok
    end,
    io:format("validating ",[]),
    case validating_parser_q(URI) of
	{Error2,_Tail2} when record(Error2,xmlElement) ->
	    io:format("ERROR ~p~n URI=~p~n",[Error2,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test});
	_Res2 ->
	    io:format("OK~n",[]),
	    ok
    end.

%%% Parsers are not required to report "errors".
test_error(URI,Data) ->
    case nonvalidating_parser(URI) of
	{'EXIT',Reason} ->
	    io:format("ERROR ~p~n URI=~p~n",[Reason,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test});
	_Res ->
	    io:format("nonvalidating OK ",[]),
	    ok
    end,
    case validating_parser(URI) of
	{'EXIT',Reason2} ->
	    io:format("ERROR ~p~n URI=~p~n",[Reason2,URI]),
	    io:format(Data),
	    throw({'EXIT', failed_test});
	_Res2 ->
	    io:format("validating OK~n",[]),
	    ok
    end.
    

%%% Use xmerl as nonvalidating XML parser
nonvalidating_parser(URI) ->
    (catch xmerl_scan:file(URI,[])).


%%% Use xmerl as nonvalidating XML parser
nonvalidating_parser_q(URI) ->
    (catch xmerl_scan:file(URI,[{quiet,true}])).


%%% Use xmerl as validating XML parser
validating_parser(URI) ->
    (catch xmerl_scan:file(URI,[{validation,true}])).


%%% Use xmerl as validating XML parser
validating_parser_q(URI) ->
    (catch xmerl_scan:file(URI,[{validation,true},{quiet,true}])).




%%% ============================================================================
%%% Callbacks for parsing of Simplified DocBook XML

para(Data,Attrs,US) ->
    case US of
	Int when integer(Int) -> Int+1;
	undefined -> 1
    end.


